"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Util_1 = require("./Util");
// 先提供一个同步版本的获取信息
class CacheMoudle {
    constructor(ttl) {
        this.valueCache = {};
        this.ttl = 0;
        this.ttl = ttl || this.ttl;
    }
    static init() {
        // 5s 检查一次
        if (this.handle) {
            this.handle.stop();
        }
        this.handle = Util_1.Util.setInterval(CacheMoudle.update, 5 * 1000);
        return Promise.resolve();
    }
    /**
     * 创建cache
     * @param name 模块名字 如果没有名字将不支持自动update功能
     * @param ttl 数据有效期
     */
    static createCache(name, ttl) {
        if (name == undefined) {
            return new CacheMoudle(ttl);
        }
        else {
            if (!this.cacheMap.hasOwnProperty(name)) {
                this.cacheMap[name] = new CacheMoudle(ttl);
            }
            return this.cacheMap[name];
        }
    }
    // 移除cache
    static removeCache(name) {
        delete this.cacheMap[name];
    }
    static update() {
        let curr = Date.now();
        for (let key in CacheMoudle.cacheMap) {
            let unit = CacheMoudle.cacheMap[key];
            unit._update(curr);
        }
        return Promise.resolve();
    }
    set(key, value) {
        if (!this.valueCache.hasOwnProperty(key)) {
            this.valueCache[key] = { time: Date.now(), value: value };
        }
        else {
            this.valueCache[key].time = Date.now();
            this.valueCache[key].value = value;
        }
    }
    hasOwnProperty(key) {
        return this.valueCache.hasOwnProperty(key);
    }
    get(key) {
        if (this.valueCache.hasOwnProperty(key))
            return this.valueCache[key].value;
        else
            return undefined;
    }
    del(key) {
        delete this.valueCache[key];
    }
    _update(curr) {
        // 增加淘汰算法空间
        if (this.ttl == 0)
            return;
        for (let key in this.valueCache) {
            let unit = this.valueCache[key];
            if (unit.time + this.ttl < curr) {
                delete this.valueCache[key];
            }
        }
    }
}
exports.CacheMoudle = CacheMoudle;
CacheMoudle.cacheMap = {};
