"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const bodyParser = require("body-parser");
const fs_1 = require("fs");
const path_1 = require("path");
const zlib_1 = require("zlib");
class cWebRouteModule {
    constructor() {
        this.usePools = [];
        this._route_path = process.cwd();
        this.inited = false;
        this.favicon_data = Buffer.from("");
    }
    use(...handlers) {
        if (this.app)
            this.app.use(...handlers);
    }
    init(port, finish) {
        if (this.inited)
            return finish();
        // 建立一个web监控
        let app = express_1.default();
        let server = app.listen(port);
        server.on("error", function (e) { console.log(e); });
        server.on("listening", () => {
            console.log("listen on:" + port);
            this.app = app;
            this.app.use(function (req, res, next) {
                if (exports.WebRouteModule.openCross) {
                    res.header("Access-Control-Allow-Origin", "*");
                    res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept");
                    res.header("Access-Control-Allow-Methods", "GET,POST,OPTIONS");
                }
                next();
            });
            this.app.options('*', function (req, res) {
                res.end();
            });
            this.app.use(bodyParser.text());
            this.app.use(bodyParser.raw());
            this.app.use(bodyParser.urlencoded({ extended: true, limit: '10mb' }));
            this.app.use(bodyParser.json());
            // 注册一下别人提交的内容
            for (let i = 0; i < this.usePools.length; i++) {
                this.app.use(this.usePools[i]);
            }
            this.app.use(this.favicon.bind(this));
            this.app.use(this.router.bind(this));
            this.app.use(function (req, res, next) {
                if (!res.headersSent) {
                    // 表示有人处理过了
                    res.writeHead(404);
                    res.write(JSON.stringify({ code: -1, errorMsg: "can not find [" + req.path + "]" }));
                }
                res.end();
            });
            finish();
        });
        this.load_favicon();
        this.inited = true;
    }
    router(req, res, next) {
        // 这里处理一下传递参数的问题统一放在 req.params 上面
        req.params = Object.assign(req.params, req.query || {}, req.body || {});
        res.setHeader('Content-Type', 'text/html; charset=utf-8');
        for (let key in req.params) {
            if (req.params[key] == 'undefined' || req.params[key] == 'null') {
                delete req.params[key];
            }
        }
        // console.log(req.url);
        // 先清理?之前的
        let sepIdx = req.url.indexOf('?');
        let tureUrl = req.url;
        if (sepIdx > 0) {
            tureUrl = req.url.slice(0, sepIdx);
        }
        let pathList = tureUrl.split('/');
        // 把里面空的都抛弃掉
        for (let i = 0; i < pathList.length; i++)
            if (!pathList[i]) {
                pathList.splice(i, 1);
                i--;
            }
        ;
        let apifile = pathList[0];
        let apifun = pathList[1];
        if (!apifile && !apifun) {
            next();
            return;
        }
        try {
            let func = require(path_1.join(this._route_path, apifile));
            if (func && apifun && func[apifun]) {
                func[apifun](req, res, next);
            }
            else if (func && func.router) {
                func.router(pathList.slice(1, pathList.length), req, res, next);
            }
            else {
                next();
            }
        }
        catch (e) {
            next();
        }
    }
    load_favicon() {
        try {
            this.favicon_data = zlib_1.gzipSync(fs_1.readFileSync(path_1.join(this._route_path, 'favicon.ico')));
        }
        catch (e) {
        }
    }
    favicon(req, res, next) {
        if (req.url == '/favicon.ico') {
            res.writeHead(200, { "Content-Type": "image/x-icon", "Content-Encoding": "gzip" });
            res.write(this.favicon_data);
            res.end();
        }
        else {
            next();
        }
    }
}
var instance = new cWebRouteModule();
/**
 * 这里负责http的解析，然后传递给对于的处理方法
 * 目前比较简陋，后续需要增加一下处理
 */
exports.WebRouteModule = {
    name: 'WebRouteModule',
    instance: instance,
    /**在模块初始化开始前注册 */
    use(...handlers) {
        if (instance.inited) {
            // console.warn('has b')
            return false;
        }
        else {
            instance.usePools.push(...handlers);
            return true;
        }
    },
    /**在模块初始化开始前注册 */
    enableStatic: function (root) {
        if (instance.inited) {
            // console.warn('has b')
            return false;
        }
        else {
            this.instance.usePools.push(express_1.default.static(root));
            return true;
        }
    },
    /**注册代码模块的位置 */
    routePath(routePath) {
        this.instance._route_path = routePath;
    },
    /**
     * 是否开启跨域
     */
    openCross: false,
    /**
     * 初始化模块
     * @param port
     */
    init(port) {
        return new Promise((resolve) => {
            instance.init(port, resolve);
        });
    }
};
